package cn.com.qmth.examcloud.core.examwork.service.impl;

import java.io.File;
import java.util.List;

import javax.persistence.CacheRetrieveMode;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.PersistenceUnit;
import javax.persistence.Query;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.helpers.poi.ExcelParser;
import cn.com.qmth.examcloud.commons.base.logging.ExamCloudLog;
import cn.com.qmth.examcloud.commons.base.logging.ExamCloudLogFactory;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.request.InsertOrUpdateStudentReq;
import cn.com.qmth.examcloud.core.basic.api.response.InsertOrUpdateStudentResp;
import cn.com.qmth.examcloud.core.examwork.dao.ExamStudentTempRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamStudentTempEntity;
import cn.com.qmth.examcloud.core.examwork.service.ExamStudentImportService;
import cn.com.qmth.examcloud.examwork.api.ExamStudentCloudService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamStudentBean;
import cn.com.qmth.examcloud.examwork.api.request.SaveExamStudentReq;
import cn.com.qmth.examcloud.examwork.api.response.SaveExamStudentResp;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年7月13日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Service
public class ExamStudentImportServiceImpl implements ExamStudentImportService {

	protected ExamCloudLog log = ExamCloudLogFactory.getLog(this.getClass());

	private static final String[] EXCEL_HEADER = new String[]{"姓名", "学号", "身份证号", "学习中心代码",
			"学习中心名称", "课程代码", "课程名称", "试卷类型", "专业", "考点", "信息采集人", "学生电话", "年级"};

	@Autowired
	ExamStudentTempRepo examStudentTempRepo;

	@Autowired
	StudentCloudService studentCloudService;

	@Autowired
	ExamStudentCloudService examStudentCloudService;

	@PersistenceUnit
	private EntityManagerFactory entityManagerFactory;

	@Override
	public void importExamStudent(File file, String fileName, Long rootOrgId, Long examId) {
		Workbook workBook = ExcelParser.getWorkBook(file, fileName);

		Sheet sheet = workBook.getSheetAt(0);

		List<String[]> lineList = ExcelParser.readSheet(sheet, 13);
		ExcelParser.close(workBook);
		
		if (CollectionUtils.isEmpty(lineList)) {
			throw new StatusException("E-651002", "Excel 没有内容");
		}

		List<ExamStudentTempEntity> list = Lists.newArrayList();
		long batchId = System.currentTimeMillis();

		for (int i = 0; i < lineList.size(); i++) {
			String[] line = lineList.get(i);
			if (0 == i) {
				checkExcelHeader(line);
				continue;
			}

			ExamStudentTempEntity es = new ExamStudentTempEntity();
			es.setRootOrgId(rootOrgId);
			es.setExamId(examId);
			es.setBatchId(batchId);
			es.setName(trimAndNullIfBlank(line[0]));
			es.setStudentCode(trimAndNullIfBlank(line[1]));
			es.setIdentityNumber(trimAndNullIfBlank(line[2]));
			es.setOrgCode(trimAndNullIfBlank(line[3]));
			es.setOrgName(trimAndNullIfBlank(line[4]));
			es.setCourseCode(trimAndNullIfBlank(line[5]));
			es.setCourseName(trimAndNullIfBlank(line[6]));
			es.setPaperType(trimAndNullIfBlank(line[7]));
			es.setSpecialtyName(trimAndNullIfBlank(line[8]));
			es.setExamSite(trimAndNullIfBlank(line[9]));
			es.setInfoCollector(trimAndNullIfBlank(line[10]));
			es.setPhone(trimAndNullIfBlank(line[11]));
			es.setGrade(trimAndNullIfBlank(line[12]));

			list.add(es);
			if (0 == i % 100) {
				examStudentTempRepo.save(list);

				list = Lists.newArrayList();
			}
		}

		examStudentTempRepo.save(list);

		examStudentTempRepo.processExamStudentTemp(batchId);

		saveExamStudents(batchId);

		delete(batchId);
	}

	private void delete(Long batchId) {
		EntityManager em = entityManagerFactory.createEntityManager();
		EntityTransaction transaction = em.getTransaction();
		Query query = em
				.createQuery("delete from ExamStudentTempEntity s where s.batchId=:batchId");
		transaction.begin();
		query.setParameter("batchId", batchId);
		query.executeUpdate();
		transaction.commit();
	}

	private void saveExamStudents(Long batchId) {
		EntityManager em = entityManagerFactory.createEntityManager();
		Query query = em.createQuery("select s from ExamStudentTempEntity s where "
				+ "s.name is not null and s.orgId is not null and s.identityNumber is not null "
				+ "and s.courseId is not null and s.batchId=:batchId", ExamStudentTempEntity.class);
		query.setHint("javax.persistence.cache.retrieveMode", CacheRetrieveMode.BYPASS);
		query.setParameter("batchId", batchId);

		@SuppressWarnings("unchecked")
		List<ExamStudentTempEntity> list = query.getResultList();

		for (ExamStudentTempEntity examStudent : list) {

			saveExamStudent(examStudent);
		}
	}

	@Transactional
	private void saveExamStudent(ExamStudentTempEntity examStudent) {
		InsertOrUpdateStudentReq saveStudentReq = new InsertOrUpdateStudentReq();
		saveStudentReq.setIdentityNumber(examStudent.getIdentityNumber());
		saveStudentReq.setName(examStudent.getName());
		saveStudentReq.setPhoneNumber(examStudent.getPhone());
		saveStudentReq.setOrgCode(examStudent.getOrgCode());
		saveStudentReq.setOrgId(examStudent.getOrgId());
		saveStudentReq.setOrgName(examStudent.getOrgName());
		saveStudentReq.setRootOrgId(examStudent.getRootOrgId());
		saveStudentReq.setStudentCode(examStudent.getStudentCode());

		InsertOrUpdateStudentResp insertOrUpdateStudentResp = studentCloudService
				.insertOrUpdateStudent(saveStudentReq);

		Long studentId = insertOrUpdateStudentResp.getStudentId();

		SaveExamStudentReq saveExamStudentReq = new SaveExamStudentReq();
		saveExamStudentReq.setCourseCode(examStudent.getCourseCode());
		saveExamStudentReq.setCourseLevel(examStudent.getCourseLevel());
		saveExamStudentReq.setCourseName(examStudent.getCourseName());
		saveExamStudentReq.setExamId(examStudent.getExamId());
		saveExamStudentReq.setIdentityNumber(examStudent.getIdentityNumber());
		saveExamStudentReq.setPaperType(examStudent.getPaperType());
		saveExamStudentReq.setRootOrgId(examStudent.getRootOrgId());
		saveExamStudentReq.setStudentCode(examStudent.getStudentCode());
		saveExamStudentReq.setStudentName(examStudent.getName());
		saveExamStudentReq.setStudentId(studentId);
		saveExamStudentReq.setInfoCollector(examStudent.getInfoCollector());
		saveExamStudentReq.setGrade(examStudent.getGrade());
		saveExamStudentReq.setExamSite(examStudent.getExamSite());
		saveExamStudentReq.setSpecialtyName(examStudent.getSpecialtyName());

		SaveExamStudentResp saveExamStudentResp = examStudentCloudService
				.saveExamStudent(saveExamStudentReq);
		ExamStudentBean examStudentBean = saveExamStudentResp.getExamStudentBean();
		log.debug("examStudentId=" + examStudentBean.getId());
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param s
	 * @return
	 */
	private String trimAndNullIfBlank(String s) {
		if (StringUtils.isBlank(s)) {
			return null;
		}
		return s.trim();
	}

	private void checkExcelHeader(String[] header) {
		for (int i = 0; i < EXCEL_HEADER.length; i++) {
			if (!EXCEL_HEADER[i].equals(header[i].trim())) {
				throw new StatusException("E-651001", "Excel 表头错误");
			}
		}
	}

}
